'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _objectWithoutProperties = require('babel-runtime/helpers/object-without-properties')['default'];

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _utilsDataStore = require('./utils/dataStore');

var _utilsDataStore2 = _interopRequireDefault(_utilsDataStore);

var _DataTablePagination = require('./DataTablePagination');

var _DataTablePagination2 = _interopRequireDefault(_DataTablePagination);

var DataTable = (function (_React$Component) {
	_inherits(DataTable, _React$Component);

	function DataTable(props) {
		_classCallCheck(this, DataTable);

		_React$Component.call(this, props);
		this.store = new _utilsDataStore2['default']();
		this.initOptions(true, props);
	}

	DataTable.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
		this.initOptions(false, nextProps);
	};

	DataTable.prototype.initOptions = function initOptions(init, props) {
		var _this = this;

		var data = props.data;
		var dataFields = props.dataFields;
		var selection = props.selection;

		var paginationProps = this.getPaginationProps(props);
		var currPage = _DataTablePagination.getActivePage(paginationProps);
		var selectedRows = [];
		var keyField = undefined;

		if (!dataFields || !Array.isArray(dataFields) || dataFields.length <= 0) {
			throw 'Error. 属性 {dataFields} 必传，且不为空数组。';
		}

		dataFields.forEach(function (field) {
			if (field.idField) {
				if (keyField) {
					throw 'Error. 属性 {dataFields.idField} 只能设一个值。';
				}
				keyField = field.name;
			}
		});

		if (!keyField) {
			throw 'Error. 属性 {dataFields.idField} 需要设一个值。';
		}

		if (data.length > 0 && selection && selection.selected) {

			switch (selection.mode) {
				case 'single':
					if (typeof selection.selected !== 'number' && typeof selection.selected !== 'string') {
						throw 'Error. 属性 {selection.mode} 为single时，{selection.selected} 的值须为 number || string';
					}
					selectedRows = [selection.selected];
					break;
				case 'multiple':
					if (!Array.isArray(selection.selected)) {
						throw 'Error. 属性 {selection.mode} 为multiple时，{selection.selected} 的值须为 array';
					}
					selectedRows = selection.selected;
					break;
			}

			//如果传递了不存在的keyFieldValue，会导致selectedRows中有多余的数据，可能会出现全选无法正确的自动选上的问题，这里对selectedRows进行过滤
			selectedRows = selectedRows.filter(function (keyFieldValue) {
				return _this.filterKeyFieldValue(keyField, keyFieldValue, data);
			});
		}

		this.store.set(data).setOptions({
			pagination: props.pagination && props.pagination.local,
			currPage: currPage,
			pageSize: paginationProps.pageSize,
			keyField: keyField
		});

		var state = {
			data: this.store.setPage(currPage).get(),
			currPage: currPage,
			selectAll: false,
			selectedRows: selectedRows
		};

		init ? this.state = state : this.setState(state);
	};

	DataTable.prototype.filterKeyFieldValue = function filterKeyFieldValue(keyField, keyFieldValue, data) {
		var result = false;
		data.forEach(function (d) {
			if (d[keyField] === keyFieldValue) {
				result = true;
				return false;
			}
		});
		return result;
	};

	DataTable.prototype.getKeyField = function getKeyField() {
		return this.store.getKeyField();
	};

	DataTable.prototype.render = function render() {
		return _react2['default'].createElement(
			'div',
			{ className: 'data-table-wapper' },
			this.renderDataTable(),
			this.renderPagination()
		);
	};

	DataTable.prototype.renderDataTable = function renderDataTable() {
		var _props = this.props;
		var striped = _props.striped;
		var bordered = _props.bordered;
		var hover = _props.hover;
		var serialNumber = _props.serialNumber;
		var dataFields = _props.dataFields;
		var data = _props.data;
		var rowTemplate = _props.rowTemplate;
		var emptyText = _props.emptyText;
		var pagination = _props.pagination;
		var selection = _props.selection;
		var children = _props.children;

		var otherProps = _objectWithoutProperties(_props, ['striped', 'bordered', 'hover', 'serialNumber', 'dataFields', 'data', 'rowTemplate', 'emptyText', 'pagination', 'selection', 'children']);

		var classes = {
			'data-table': true,
			'table': true,
			'table-striped': striped,
			'table-bordered': bordered,
			'table-hover': hover
		};

		return _react2['default'].createElement(
			'table',
			_extends({}, otherProps, {
				ref: 'table',
				className: _classnames2['default'](classes, this.props.className) }),
			this.renderTableHead(),
			this.renderTableBody()
		);
	};

	DataTable.prototype.renderTableHead = function renderTableHead() {
		var _props2 = this.props;
		var serialNumber = _props2.serialNumber;
		var serialNumberHead = _props2.serialNumberHead;
		var dataFields = _props2.dataFields;
		var selection = _props2.selection;

		return _react2['default'].createElement(
			'thead',
			null,
			_react2['default'].createElement(
				'tr',
				null,
				this.renderSelectionColumn(true, 'all'),
				this.renderSerialNumberColumn(true, serialNumberHead),
				dataFields.map(function (field) {
					return field.idField ? null : _react2['default'].createElement(
						'th',
						{ key: field.name || 'cell-custom', className: field.name ? '' : 'cell-custom' },
						field.text
					);
				})
			)
		);
	};

	DataTable.prototype.renderSelectionColumn = function renderSelectionColumn(isHead, key) {
		var selection = this.props.selection;

		var ElementType = isHead ? 'th' : 'td';
		var classes = 'cell-selection';
		var checked = isHead ? this.state.selectAll : this.state.selectedRows.indexOf(key) !== -1;

		if (selection) {

			switch (selection.mode) {
				case 'single':
					return _react2['default'].createElement(
						ElementType,
						{ className: classes },
						' ',
						isHead ? null : _react2['default'].createElement('input', { key: key, type: 'radio', checked: checked, onChange: this.handleCheckInput.bind(this) })
					);
				case 'multiple':
					return _react2['default'].createElement(
						ElementType,
						{ className: classes },
						_react2['default'].createElement('input', { key: key, type: 'checkbox', checked: checked, onChange: isHead ? this.handleSelectAll.bind(this) : this.handleCheckInput.bind(this) })
					);
				default:
					throw 'Error. 属性 {selection.mode} 必须为single || multiple.';
			}
		}

		return null;
	};

	DataTable.prototype.handleSelectAll = function handleSelectAll(event) {
		var selection = this.props.selection;

		var checked = event.currentTarget.checked;
		var keyField = this.getKeyField();

		this.setState({
			selectAll: checked,
			selectedRows: checked ? this.state.data.map(function (item) {
				return item[keyField];
			}) : []
		});

		selection.onSelectAll && selection.onSelectAll(checked);
	};

	DataTable.prototype.handleSelectRow = function handleSelectRow(event) {
		var selection = this.props.selection;

		var keyField = this.getKeyField();

		if (selection && selection.clickToSelect === true) {
			var rowIndex = event.currentTarget.rowIndex - 1,
			    rowData = this.state.data[rowIndex],
			    checked = this.state.selectedRows.indexOf(rowData[keyField]) === -1;
			this.handleToggleInput(rowIndex, checked);
		}
	};

	DataTable.prototype.handleCheckInput = function handleCheckInput(event) {
		var target = event.currentTarget,
		    checked = target.checked,
		    rowIndex = target.parentElement.parentElement.rowIndex - 1; //tr.rowIndex会把<thead><tr></tr></thead>的行也计算在内

		this.handleToggleInput(rowIndex, checked);
	};

	DataTable.prototype.handleToggleInput = function handleToggleInput(rowIndex, checked) {
		var selection = this.props.selection;var data = this.state.data;var rowData = data[rowIndex];var keyField = this.getKeyField();
		var selectedRows = [];

		if (selection.mode === 'single') {
			selectedRows.push(rowData[keyField]);
		} else {
			selectedRows = this.state.selectedRows.slice();
			checked ? selectedRows.push(rowData[keyField]) : selectedRows = selectedRows.filter(function (item) {
				return item !== rowData[keyField];
			});
		}

		this.setState({
			selectAll: selectedRows.length === data.length,
			selectedRows: selectedRows
		});

		selection.onSelect && selection.onSelect(checked, rowData);
	};

	DataTable.prototype.renderSerialNumberColumn = function renderSerialNumberColumn(isHead, cellText) {
		var ElementType = isHead ? 'th' : 'td';
		return this.props.serialNumber ? _react2['default'].createElement(
			ElementType,
			{ key: 'no', className: 'cell-no' },
			cellText
		) : null;
	};

	DataTable.prototype.renderTableBody = function renderTableBody() {
		var _this2 = this;

		var _props3 = this.props;
		var serialNumber = _props3.serialNumber;
		var dataFields = _props3.dataFields;
		var rowTemplate = _props3.rowTemplate;
		var emptyText = _props3.emptyText;
		var selection = _props3.selection;

		var data = this.state.data;

		if (!data || !Array.isArray(data) || data.length <= 0) {
			return this.renderEmptyText();
		}

		if (rowTemplate) {
			return this.renderTableBodyForRowTemplate();
		}

		var rowClasses = {
			'row-selection': selection && selection.clickToSelect
		};

		var keyField = this.getKeyField();

		return _react2['default'].createElement(
			'tbody',
			null,
			data.map(function (item, index) {
				return _react2['default'].createElement(
					'tr',
					{ className: _classnames2['default'](rowClasses), key: 'row' + index, onClick: _this2.handleSelectRow.bind(_this2) },
					_this2.renderSelectionColumn(false, item[keyField]),
					_this2.renderSerialNumberColumn(false, ++index),
					dataFields.map(function (field) {
						return field.idField ? null : _react2['default'].createElement(
							'td',
							{ key: field.name || 'cell-custom', className: field.name ? '' : 'cell-custom' },
							item[field.name] || field.value || ''
						);
					})
				);
			})
		);
	};

	DataTable.prototype.renderEmptyText = function renderEmptyText() {
		var _props4 = this.props;
		var dataFields = _props4.dataFields;
		var serialNumber = _props4.serialNumber;
		var selection = _props4.selection;
		var emptyText = _props4.emptyText;

		var columns = dataFields.length;

		// serialNumber && ++columns
		selection && ++columns;

		return _react2['default'].createElement(
			'tbody',
			null,
			_react2['default'].createElement(
				'tr',
				null,
				_react2['default'].createElement(
					'td',
					{ colSpan: columns, className: 'empty-text' },
					emptyText
				)
			)
		);
	};

	DataTable.prototype.renderTableBodyForRowTemplate = function renderTableBodyForRowTemplate() {
		var _this3 = this;

		var _props5 = this.props;
		var serialNumber = _props5.serialNumber;
		var dataFields = _props5.dataFields;
		var rowTemplate = _props5.rowTemplate;
		var selection = _props5.selection;

		var data = this.state.data;

		//校验模板是否为 <tr><td></td></tr> 格式
		if (rowTemplate.type !== 'tr') {
			throw 'Error. 行模板 {rowTemplate} 必须为<tr><td></td></tr>元素';
		}

		var cells = rowTemplate.props.children;

		//校验模板中，其TD数量是否和dataFields长度一致
		if (!cells || cells.length !== dataFields.length) {
			throw 'Error. 模板的列数必须和属性 {dataFields} 对应上';
		}

		cells.forEach(function (cell) {
			if (cell.type !== 'td') {
				throw 'Error. 行模板 {rowTemplate} 必须为<tr><td></td></tr>元素';
			}
		});

		var rowClasses = {
			'row-selection': selection && selection.clickToSelect
		};

		var renderChildrenNode = function renderChildrenNode(parent, child, data) {
			if (typeof child === 'string') {
				//当children为string时，把数据占位符%%替换成真实数据
				return child.replace(/%\w*%/g, function (key) {
					return data[key.replace(/%/g, '')] || '';
				});
			} else if (_react.isValidElement(child)) {
				//child为element时，递归调用renderChildrenNode，对所有children的%%数据占位符替换成真实数据。然后return clone后的element
				var node = renderChildrenNode(child, child.props.children, data);
				return _react.cloneElement(child, { children: node });
			}

			return child;
		};

		var keyField = this.getKeyField();

		return _react2['default'].createElement(
			'tbody',
			null,
			data.map(function (dataItem, rowIndex) {
				return _react2['default'].createElement(
					'tr',
					{ className: _classnames2['default'](rowClasses), key: 'row' + rowIndex, onClick: _this3.handleSelectRow.bind(_this3) },
					_this3.renderSelectionColumn(false, dataItem[keyField]),
					_this3.renderSerialNumberColumn(false, ++rowIndex),
					dataFields.map(function (field, cellIndex) {
						var _cells$cellIndex$props = cells[cellIndex].props;
						var children = _cells$cellIndex$props.children;
						var childrenNode = _cells$cellIndex$props.childrenNode;
						var className = _cells$cellIndex$props.className;

						var otherProps = _objectWithoutProperties(_cells$cellIndex$props, ['children', 'childrenNode', 'className']);

						if (field.idField === true) {
							//idField为ID标识列，这里直接返回null，暂不作其它处理
							return null;
						}

						if (childrenNode && typeof childrenNode !== 'function') {
							throw 'Error. TD的属性 {childrenNode} 必须为function';
						}

						// field.name 不存在时，给td添加 cell-custom 样式
						var cellClasses = { 'cell-custom': !field.name };

						var column = _react2['default'].createElement('td', _extends({}, otherProps, { key: field.name || 'cell-custom', className: _classnames2['default'](cellClasses, className) }));

						//如果该列传递了childrenNode，直接调用它得到TD的children
						var columnChildren = childrenNode ? childrenNode(dataItem || {}) : renderChildrenNode(column, children, dataItem || {});

						return _react.cloneElement(column, { children: columnChildren });
					})
				);
			})
		);
	};

	DataTable.prototype.renderPagination = function renderPagination() {
		var pagination = this.props.pagination;

		return pagination ? _react2['default'].createElement(_DataTablePagination2['default'], _extends({}, pagination, {
			activePage: this.state.currPage,
			onChangePage: pagination.local ? this.handleChangePage.bind(this) : pagination.onChangePage })) : null;
	};

	DataTable.prototype.handleChangePage = function handleChangePage(event, selectedEvent) {
		var currPage = selectedEvent.eventKey;

		this.setState({
			data: this.store.setPage(currPage).get(),
			currPage: currPage
		});
	};

	DataTable.prototype.getPaginationProps = function getPaginationProps(props) {
		return (props || this.props).pagination || _DataTablePagination2['default'].defaultProps;
	};

	/**
  * 获取当前页已选择的行数据
  * 该方法如果在this.props.selection.onSelect()中即时调用，由于state还没更新，这里的数据将会是历史的脏数据
  */

	DataTable.prototype.getSelectedDatas = function getSelectedDatas() {
		var selectedRows = this.state.selectedRows;

		var keyField = this.getKeyField();
		return this.store.getAllData().filter(function (item) {
			return selectedRows.indexOf(item[keyField]) !== -1;
		});
	};

	return DataTable;
})(_react2['default'].Component);

DataTable.propTypes = {
	/**
  * 是否隔行着色
  */
	striped: _react.PropTypes.bool,
	/**
  * 是否显示border
  */
	bordered: _react.PropTypes.bool,
	/**
  * 是否在hover时，改变行背景色
  */
	hover: _react.PropTypes.bool,
	/**
  * 是否添加序号列
  */
	serialNumber: _react.PropTypes.bool,
	/**
  * 序号列头显示的TEXT
  */
	serialNumberHead: _react.PropTypes.string,
	/**
  * 用于解析data数据的field标识
  */
	dataFields: _react.PropTypes.array.isRequired,
	/**
  * 用于生成DataTable的源数据
  */
	data: _react.PropTypes.array,
	/**
  * 用于生成DataTable的行模板，格式为 &lt;tr&gt;&lt;td&gt;&lt;/td&gt;&lt;/tr&gt;
  */
	rowTemplate: _react.PropTypes.element,
	/**
  * 没有数据时，显示的提示信息，string || element
  */
	emptyText: _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.element]),
	/**
  * 是否添加分页条
  * 
  * pagination: {
  *   ellipsis: bool,
  *   boundaryLinks: bool,
  *   maxButtons: number,
  *   activePage: number,
  *   pageSize: number,
  *   total: number,
  *   showTotalText: bool,
  *   align: 'right' || 'left',
  *   local: bool,
  *   onChangePage: func
  * }
  */
	pagination: _react.PropTypes.object,
	/**
  * 是否添加行选择，single或multiple
  * 
  * selection: {
  *   mode: 'single' || 'multiple',
  *   selected: number || string || array,
  *   clickToSelect: bool,
  *   onSelect: func,
  *   onSelectAll: func
  * }
  */
	selection: _react.PropTypes.object
};

DataTable.defaultProps = {
	data: [],
	striped: false,
	bordered: true,
	hover: false,
	serialNumber: false,
	serialNumberHead: '序号',
	emptyText: '没有可显示的数据'
};

exports['default'] = DataTable;
module.exports = exports['default'];